
#include <windows.h>  
#include "create.h"  
#include "commctrl.h"


#if defined (WIN32)
	#define IS_WIN32 TRUE
#else
	#define IS_WIN32 FALSE
#endif

#define IS_NT      IS_WIN32 && (BOOL)(GetVersion() < 0x80000000)
#define IS_WIN32S  IS_WIN32 && (BOOL)(!(IS_NT) && (LOBYTE(LOWORD(GetVersion()))<4))
#define IS_WIN95   (BOOL)(!(IS_NT) && !(IS_WIN32S)) && IS_WIN32

HINSTANCE hInst;   // current instance

LPCTSTR lpszAppName = "MyApp";
LPCTSTR lpszTitle   = "ImageList_Create()"; 


BOOL RegisterWin95( CONST WNDCLASS* lpwc );


int APIENTRY WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance,
                      LPTSTR lpCmdLine, int nCmdShow)
{
   MSG      msg;
   HWND     hWnd; 
   WNDCLASS wc;

   wc.style         = CS_HREDRAW | CS_VREDRAW;
   wc.lpfnWndProc   = (WNDPROC)WndProc;       
   wc.cbClsExtra    = 0;                      
   wc.cbWndExtra    = 0;                      
   wc.hInstance     = hInstance;              
   wc.hIcon         = LoadIcon (hInstance, lpszAppName); 
   wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
   wc.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);
   wc.lpszMenuName  = lpszAppName;              
   wc.lpszClassName = lpszAppName;              

   if ( IS_WIN95 )
   {
      if ( !RegisterWin95( &wc ) )
         return( FALSE );
   }
   else if ( !RegisterClass( &wc ) )
      return( FALSE );

   hInst = hInstance; 

   hWnd = CreateWindow( lpszAppName, 
                        lpszTitle,    
                        WS_OVERLAPPEDWINDOW, 
                        CW_USEDEFAULT, 0, 
                        CW_USEDEFAULT, 0,  
                        NULL,              
                        NULL,              
                        hInstance,         
                        NULL               
                      );

   if ( !hWnd ) 
      return( FALSE );

   ShowWindow( hWnd, nCmdShow ); 
   UpdateWindow( hWnd );         

   while( GetMessage( &msg, NULL, 0, 0) )   
   {
      TranslateMessage( &msg ); 
      DispatchMessage( &msg );  
   }

   return( msg.wParam ); 
}


BOOL RegisterWin95( CONST WNDCLASS* lpwc )
{
   WNDCLASSEX wcex;

   wcex.style         = lpwc->style;
   wcex.lpfnWndProc   = lpwc->lpfnWndProc;
   wcex.cbClsExtra    = lpwc->cbClsExtra;
   wcex.cbWndExtra    = lpwc->cbWndExtra;
   wcex.hInstance     = lpwc->hInstance;
   wcex.hIcon         = lpwc->hIcon;
   wcex.hCursor       = lpwc->hCursor;
   wcex.hbrBackground = lpwc->hbrBackground;
   wcex.lpszMenuName  = lpwc->lpszMenuName;
   wcex.lpszClassName = lpwc->lpszClassName;

   // Added elements for Windows 95.
   //...............................
   wcex.cbSize = sizeof(WNDCLASSEX);
   wcex.hIconSm = LoadImage(wcex.hInstance, lpwc->lpszClassName, 
                            IMAGE_ICON, 16, 16,
                            LR_DEFAULTCOLOR );
			
   return RegisterClassEx( &wcex );
}


#define NODRAG    0
#define POSSIBLE  1
#define DRAGING   2

LRESULT CALLBACK WndProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
static POINT      pt, ptWnd, movePt;
static RECT       rectangle;
static HIMAGELIST hList     = NULL;
static int        nDraging  = NODRAG;
static int        nSel      = -1;

   switch( uMsg )
   {
      case WM_CREATE :
              InitCommonControls();

              // Create the image list.
              //.......................
              hList = ImageList_Create( 32, 32, ILC_COLOR | ILC_MASK, 3, 1 );
              if ( hList )
              {
                 HBITMAP hBmp, hMask;
                 
                 // Add masked image.
                 //..................
                 hBmp  = LoadBitmap( hInst, "BMP1" );
                 hMask = LoadBitmap( hInst, "BMP1MSK" );
                 ImageList_Add( hList, hBmp, hMask );
                 DeleteObject( hBmp );
                 DeleteObject( hMask );

                 // Add an image and build a mask.
                 //...............................
                 hBmp = LoadBitmap( hInst, "BMP2" );
                 ImageList_AddMasked( hList, hBmp, RGB( 0, 255, 0 ) );
                 DeleteObject( hBmp );

                 // Add an the arrow cursor.
                 //.........................
                 ImageList_AddIcon( hList, (HICON)LoadCursor( NULL, IDC_ARROW ) );

                 rectangle.left   = 10;
                 rectangle.top    = 80;
                 rectangle.right  = 250;
                 rectangle.bottom = 160;
              }
              break;

      case WM_PAINT :
              {
                 PAINTSTRUCT ps;

                 // Paint the two images on the client area.
                 //.........................................
                 BeginPaint( hWnd, &ps );
                 ImageList_Draw( hList, 0, ps.hdc, 10, 10, ILD_NORMAL | (nSel == 0 ? ILD_SELECTED : 0));
                 ImageList_Draw( hList, 1, ps.hdc, 60, 10, ILD_NORMAL | (nSel == 1 ? ILD_SELECTED : 0));
                 Rectangle( ps.hdc, rectangle.left, rectangle.top, rectangle.right, rectangle.bottom );
                 TextOut( ps.hdc, rectangle.left+10, rectangle.top+10, "Drag cursor will not show here.", 31 );

                 // If the image is being dragged, get the drag image
                 // and display it within the rectanlge on the client area.
                 //........................................................
                 if ( nDraging == DRAGING )
                 {
                    HIMAGELIST hDragList = ImageList_GetDragImage( NULL, NULL );
                    int        cx,cy;

                    ImageList_GetIconSize( hDragList, &cx, &cy );
                    ImageList_Draw( hDragList, 0, ps.hdc, 
                                    rectangle.left+((rectangle.right-rectangle.left-cx)/2), 
                                    rectangle.top+24, ILD_NORMAL ); 
                 }
                 EndPaint( hWnd, &ps );
              }
              break;

      case WM_LBUTTONDOWN :
              {
                 RECT  rect;

                 rect.top    = 10;
                 rect.bottom = 42;
                 rect.right  = 42;
                 rect.left   = 10;

                 pt.x = LOWORD( lParam );
                 pt.y = HIWORD( lParam );

                 nSel = -1;

                 // Check which image the user selected.
                 //.....................................
                 if ( PtInRect( &rect, pt ) )
                    nSel = 0;

                 rect.right = 82;
                 rect.left  = 50;
                 if ( PtInRect( &rect, pt ) )
                    nSel = 1;

                 InvalidateRect( hWnd, NULL, TRUE );

                 // Calculate the offsets from the main window to the client.
                 //..........................................................
                 if ( nSel > -1 )
                 {
                    nDraging = POSSIBLE;
                    SetCapture( hWnd );

                    GetWindowRect( hWnd, &rect );
                    ptWnd.x = rect.left;
                    ptWnd.y = rect.top;

                    ScreenToClient( hWnd, &ptWnd );
                    ptWnd.y += GetSystemMetrics( SM_CYSIZEFRAME )+3;
                    ptWnd.x += GetSystemMetrics( SM_CXSIZEFRAME )+2;
                 }
              }
              break;

      case WM_LBUTTONUP :
              // If draging, Finish.
              //....................
              if ( nDraging == DRAGING ) 
              {
                 ImageList_DragLeave( hWnd );
                 ImageList_EndDrag();
                 ShowCursor( TRUE );
              }

              nDraging = NODRAG;
              ReleaseCapture();
              InvalidateRect( hWnd, &rectangle, TRUE );
              break;

      case WM_MOUSEMOVE :
              if ( nDraging == POSSIBLE && 
                   ( abs(LOWORD( lParam ) - pt.x) > 2 ||
                     abs(HIWORD( lParam ) - pt.y) > 2 ) )
              {
                 nDraging = DRAGING;

                 // Hide the mouse cursor.
                 //.......................
                 ShowCursor( FALSE );

                 // Start Draging.
                 //...............
                 ImageList_BeginDrag( hList, nSel, pt.x-(15+nSel*50), pt.y-15 );

                 // Add the arrow to the drag cursor.
                 //..................................
                 ImageList_SetDragCursorImage( hList, 2, pt.x-(15+nSel*50), pt.y-15 );

                 // Show the drag cursor.
                 //......................
                 ImageList_DragEnter( hWnd, LOWORD( lParam )-ptWnd.x, HIWORD( lParam )-ptWnd.y );
                 InvalidateRect( hWnd, &rectangle, TRUE );
              }

              // If draging, move the drag cursor.
              //..................................
              if ( nDraging == DRAGING )
              {
                 movePt.x = LOWORD( lParam );
                 movePt.y = HIWORD( lParam );
                 if ( PtInRect( &rectangle, movePt ) )
                    ImageList_DragShowNolock( FALSE );
                 else
                    ImageList_DragShowNolock( TRUE );

                 ImageList_DragMove( LOWORD( lParam )-ptWnd.x, HIWORD( lParam )-ptWnd.y );
              }
              break;


      case WM_COMMAND :
              switch( LOWORD( wParam ) )
              {
                 case IDM_ABOUT :
                        DialogBox( hInst, "AboutBox", hWnd, (DLGPROC)About );
                        break;

                 case IDM_EXIT :
                        DestroyWindow( hWnd );
                        break;
              }
              break;
      
      case WM_DESTROY :
              if ( hList )
                 ImageList_Destroy( hList );

              PostQuitMessage(0);
              break;

      default :
            return( DefWindowProc( hWnd, uMsg, wParam, lParam ) );
   }

   return( 0L );
}


LRESULT CALLBACK About( HWND hDlg,           
                        UINT message,        
                        WPARAM wParam,       
                        LPARAM lParam)
{
   switch (message) 
   {
       case WM_INITDIALOG: 
               return (TRUE);

       case WM_COMMAND:                              
               if (   LOWORD(wParam) == IDOK         
                   || LOWORD(wParam) == IDCANCEL)    
               {
                       EndDialog(hDlg, TRUE);        
                       return (TRUE);
               }
               break;
   }

   return (FALSE); 
}
